const char *softAP_ssid;
const char *softAP_password = "";
extern "C" {
#include "user_interface.h"
}

char *myHostname;

ESP8266WebServer server(80);

IPAddress apIP(192, 168, 4, 1);
IPAddress netMsk(255, 255, 255, 0);

boolean connect;
long lastConnectTry = 0;

int status = WL_IDLE_STATUS;

void loopWifi()
{
  webSocket.loop();
  if (connect) {
    Serial.println ( "Connect requested" );
    connect = false;
    Serial.println("Connecting as wifi client...");
    WiFi.disconnect();
    WiFi.begin ( ssid, password );
  }

    int s = WiFi.status();

    if (status != s) { // WLAN status change
      Serial.print ( "Status: " );
      Serial.println ( s );
      status = s;
      if (s == WL_CONNECTED) {
        Serial.println ( "" );
        Serial.print ( "Connected to " );
        Serial.println ( ssid );
        Serial.print ( "IP address: " );
        Serial.println ( WiFi.localIP() );
      } else if (s == WL_NO_SSID_AVAIL) {
        WiFi.disconnect();
      }
    }

  server.handleClient();
}

void setupWifi()
{
  WiFi.softAPConfig(apIP, apIP, netMsk);
  String tmpId = name + getUniqueId();
  myHostname = (char*)malloc(tmpId.length() + 1);
  tmpId.toCharArray(myHostname, tmpId.length() + 1);
  WiFi.softAP(myHostname, softAP_password);
  delay(500);
  server.on("/", handleRoot);
  server.begin();
  connect = strlen(ssid) > 0; // Request WLAN connect if there is a SSID
  webSocket.begin();
  webSocket.onEvent(webSocketEvent);
}

void sendScript(String _server)
{
  server.sendContent(
    "\
<script language = 'javascript' type = 'text/javascript'>\
var wsUri = 'ws://' + window.location.hostname + ':81/';\
\
\
  function doWebSocket() {\
      websocket = new WebSocket(wsUri);\
      websocket.onopen = function(evt) {\
          console.log('CONNECTED');\
      };\
      websocket.onclose = function(evt) {\
          console.log('DISCONNECTED');\
          doWebSocket();\
      };\
      websocket.onmessage = function(evt) {\
        console.log('Got message:' + evt.data)\
      };\
   }\
\
  window.addEventListener('load', doWebSocket, false);\
    </script>\
  ");
}

void sendSlider(String _name, int _id) {
  server.sendContent(String("<div>" +_name + ":<br>" +\
   "<input type='range' onChange='websocket.send(\"S" + _id +  "\"+this.value);\
   ' id='" + _name + "' min='0' value='" + sliderValues[_id] + "' max='255'></div>" +\
   "<script language = 'javascript' type = 'text/javascript'>\
   document.getElementById('"+_name+"').addEventListener('input',\  
   function(){websocket.send(\"S" + _id +  "\"+this.value)}, false);</script>"));
}

void handleRoot() {
  if (captivePortal()) {
    return;
  }
  
  server.setContentLength(1500);
  server.send(200, "text/html", ""); // Empty content inhibits Content-length header so we have to close the socket ourselves.
  server.sendContent("<html><head><style>body{line-height: 1.5;\
    background-color:black; color:white}</style></head>");
  server.sendContent("<body><meta name=\"viewport\" content=\"width=device-width, initial-scale=2\">");
  sendScript(IpAddress2String(WiFi.softAPIP()));
  
  server.sendContent("<h1>RGB:</h1>" );
 
  sendSlider("Red",0);
  sendSlider("Green",1);
  sendSlider("Blue",2);
 
  server.sendContent("</body></html>");
//  server.client().stop(); // Stop is needed because we sent no content length
}

boolean captivePortal() {
  if (!isIp(server.hostHeader()) && !(server.hostHeader().equalsIgnoreCase((String(myHostname) + ".local")))) {
    server.client().stop(); // Stop is needed because we sent no content length
    return true;
  }
  return false;
}

String IpAddress2String(const IPAddress& ipAddress)
{
  return String(ipAddress[0]) + String(".") + \
         String(ipAddress[1]) + String(".") + \
         String(ipAddress[2]) + String(".") + \
         String(ipAddress[3])  ;
}

String getUniqueId()
{
  uint8_t MAC_array[6];
  String tmpId = "";
  WiFi.macAddress(MAC_array);
  for (int i = 0; i < sizeof(MAC_array); ++i) {
    tmpId = tmpId + (char)(MAC_array[i] % 23 + 97);
  }
  return tmpId;
}

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length) {

  switch (type) {
    case WStype_CONNECTED:
      {
        IPAddress ip = webSocket.remoteIP(num);
        Serial.printf("[%u] Websocket Connected from %d.%d.%d.%d url: %s\n", num, ip[0], ip[1], ip[2], ip[3], payload);
        webSocket.sendTXT(num, " Connected");
      }
      break;
    case WStype_TEXT:          
      if (payload[0] == 'S')
      {
        sliderValues[String((char *)payload).substring(1,2).toInt()]\
        = String((char *)payload).substring(2).toInt();
      }
      break;
     }
}

boolean isIp(String str) {
  for (int i = 0; i < str.length(); i++) {
    int c = str.charAt(i);
    if (c != '.' && (c < '0' || c > '9')) {
      return false;
    }
  }
  return true;
}

String toStringIp(IPAddress ip) {
  String res = "";
  for (int i = 0; i < 3; i++) {
    res += String((ip >> (8 * i)) & 0xFF) + ".";
  }
  res += String(((ip >> 8 * 3)) & 0xFF);
  return res;
}
